//----------------------------------------------------------------------------
// pacman-like game
// copyright 2010 christopher rasmussen
// university of delaware
// v. 1.01
// * attempt_move() function added

//----------------------------------------------------------------------------

#include "Agent.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// ALL STUDENTS WRITE THIS FOR LAB 7

// breadth-first search: mark all block vertices with 
// minimum distance to start location

void BFS(Point_2D start, BlockMaze *M)
{

}

//----------------------------------------------------------------------------

// ALL STUDENTS WRITE THIS FOR LAB 7

// assuming BFS() has been called and distances set,
// works backward from t to distance 0 vertex (aka the start)
// and fills P with sequence

void compute_BFS_path(Point_2D target, vector <Point_2D> & P, BlockMaze *M)
{

}

//----------------------------------------------------------------------------

// HONORS STUDENTS WRITE THIS FOR LAB 7

void Dijkstra(Point_2D start, BlockMaze *M)
{

}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// ALL STUDENTS WRITE THIS FOR LAB 7

// this should compute a path to the "next" power pill, then take ONE step in that
// direction, until there are no more power pills.  see handle_key() below for
// how runner moves

void RunnerAgent::update(GameState & GS, BlockMaze *M)
{

}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

Agent::Agent(string na, char glyph, int color, int initial_x, int initial_y)
{
  name = na;
  position.x = initial_x;
  position.y = initial_y;

  terrain_wait_counter = 0;

  draw_glyph = glyph;
  draw_color = color;
  do_draw_path = false;
  do_draw_distances = false;
  do_draw_name = false;
}

//----------------------------------------------------------------------------

// this is the only way an agent should move--DO NOT SET POSITION DIRECTLY!!!
// also, there should only be one call to attempt_move() per call to update().

bool Agent::attempt_move(int row, int col, BlockMaze *M)
{
  // is move legal (to interior of maze, no diagonals, no hopping or teleporting)?
  
  if (row <= 0 || row >= M->num_rows - 1 || col <= 0 || col >= M->num_cols - 1)
    return false;

  if (!((col == position.col + 1 && row == position.row) || 
	(col == position.col - 1 && row == position.row) ||
	(col == position.col && row == position.row + 1) ||
	(col == position.col && row == position.row - 1)))
    return false;

  // is move to an empty space?

  if (M->vertex[row][col].is_wall())
    return false;

  // does terrain allow it right now (i.e, agent may have to "wait" a few turns)

  if (terrain_wait_counter > 0) 
    return false;

  // move is approved...

  // ...if destination is terrain, set wait counter

  if (M->vertex[row][col].is_water())
    terrain_wait_counter = WATER_COST;
  else if (M->vertex[row][col].is_fire())
    terrain_wait_counter = FIRE_COST;

  // ...ok---actually do it now!
  
  position.col = col;
  position.row = row;

  return true;
}

//----------------------------------------------------------------------------

// react to keyboard input

void Agent::handle_key(char c, BlockMaze *M)
{
  switch (c) {
    
    // move up	
      
  case NORTH_KEY:
    attempt_move(position.y - 1, position.x, M);
    break;
    
    // move down
    
  case SOUTH_KEY:
    attempt_move(position.y + 1, position.x, M);
    break;

    // move left

  case WEST_KEY:
    attempt_move(position.y, position.x - 1, M);
    break;

    // move right

  case EAST_KEY:
    attempt_move(position.y, position.x + 1, M);
    break;

    // change various drawing debug modes

  case DRAW_DISTANCE_KEY:
    do_draw_distances = !do_draw_distances;
    break;

  case DRAW_PATH_KEY:
    do_draw_path = !do_draw_path;
    break;

  case DRAW_NAME_KEY:
    do_draw_name = !do_draw_name;
    break;
  }

}

//----------------------------------------------------------------------------

// draw optional underlay of distances/path/name, etc. to
// help with debugging or to clarify agent's internal state

void Agent::draw_debug(BlockMaze *M)
{
  // BFS output -- note that distances in M "belong" to last
  // agent to run BFS and not necessarily this one

  if (do_draw_distances)
    M->draw_distances();

  // path to target in agent color

  attron(COLOR_PAIR(draw_color));  

  if (do_draw_path) {
    for (int i = 0; i < path.size(); i++) {
      move(path[i].y, path[i].x);
      printw("*");
    }
  }

  attroff(COLOR_PAIR(draw_color));  

  // name?

  if (do_draw_name) {
    move(position.y, position.x + 1);
    printw("%s", name.c_str());
  }

}

//----------------------------------------------------------------------------

// draw the agent itself--always on top of debug information, even from
// other agents

void Agent::draw(BlockMaze *M)
{
  attron(COLOR_PAIR(draw_color));  

  move(position.y, position.x);
  printw("%c", draw_glyph);

  attroff(COLOR_PAIR(draw_color));  
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// "compress" information about variable parts of game state--where agents are, where
// remaining power pills are, etc., into single class object

void GameState::update(vector <Agent *> & agent_list, vector <Point_2D> & power_pill_list)
{
  int i;

  object_position.clear();
  object_name.clear();

  for (i = 0; i < agent_list.size(); i++) {
    object_position.push_back(agent_list[i]->position);
    object_name.push_back(agent_list[i]->name);
  }

  for (i = 0; i < power_pill_list.size(); i++) {
    object_position.push_back(power_pill_list[i]);
    object_name.push_back("power pill");
  }
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

