#ifndef AGENT_HH
#define AGENT_HH

//----------------------------------------------------------------------------
// pacman-like game
// copyright 2010 christopher rasmussen
// university of delaware
// v. 1.01
// * attempt_move() added

//----------------------------------------------------------------------------

#include "BlockMaze.hh"
#include "Utils.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

void BFS(Point_2D, BlockMaze *);
void Dijkstra(Point_2D, BlockMaze *);
void BFS_expand(Point_2D, Point_2D &, queue <Point_2D> &, BlockMaze *);
void compute_BFS_path(Point_2D, vector <Point_2D> &, BlockMaze *);

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

class Agent;

class GameState
{
public:
  GameState() { object_position.clear(); object_name.clear(); }

  void update(vector <Agent *> &, vector <Point_2D> &);

  vector <Point_2D> object_position;
  vector <string> object_name;
};

//----------------------------------------------------------------------------

const char NORTH_KEY     = 'w';
const char SOUTH_KEY     = 's';
const char WEST_KEY      = 'a';
const char EAST_KEY      = 'd';

const char DRAW_DISTANCE_KEY  = 'o';
const char DRAW_PATH_KEY      = 'p';
const char DRAW_NAME_KEY      = 'n';

//----------------------------------------------------------------------------

class Agent
{
public:

  Agent(string, char, int, int = 0, int = 0);
  virtual void update(GameState &, BlockMaze *) = 0;
  void draw(BlockMaze *);
  void draw_debug(BlockMaze *);
  void handle_key(char, BlockMaze *);
  bool attempt_move(int, int, BlockMaze *);

  // key identifiers

  string name;
  Point_2D position;

  int terrain_wait_counter;

  // display variables

  char draw_glyph;
  int draw_color;

  bool do_draw_distances;
  bool do_draw_path;
  bool do_draw_name;

  // optional "AI" variables and data structures

  vector <Point_2D> path;

};

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

class RunnerAgent : public Agent
{
public:

  RunnerAgent(string s, char c, int i, int x = 0, int y = 0) : Agent(s, c, i, x, y) { }
  ~RunnerAgent() { }

  void update(GameState &, BlockMaze *);
};

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#endif
